@@ -17,8 +17,10 @@ setMethod(
             cat("Log-likelihood = ", x@logLik, ifelse(length(x@SElogLik) > 0, 
                                                                paste(', SE = ', round(x@SElogLik,3)),
                                                                ''), "\n",sep='')			
-            cat("AIC =", x@AIC, "\n")			
+            cat("AIC =", x@AIC, "\n")	
+            cat("AICc =", x@AICc, "\n")
             cat("BIC =", x@BIC, "\n")
+            cat("SABIC =", x@SABIC, "\n")
             if(!is.nan(x@p))
                 cat("G^2 = ", round(x@G2,2), ", df = ", 
                     x@df, ", p = ", round(x@p,4), "\nTLI = ", round(x@TLI,3),
@@ -179,15 +181,22 @@ setMethod(
             object <- object2
             object2 <- tmp
         }
-        X2 <- 2*object2@logLik - 2*object@logLik 
-        AICdiff <- object@AIC - object2@AIC  
-        BICdiff <- object@BIC - object2@BIC  
-        se <- round(object@SElogLik + object2@SElogLik,3)
-        cat("\nChi-squared difference: \n\nX2 = ", round(X2,3), 
-            ", df = ", df, ", p = ", round(1 - pchisq(X2,abs(df)),4), 
-            "\n", sep="")
-        cat("AIC difference = ", round(AICdiff,3),"\n", sep='')
-        cat("BIC difference = ", round(BICdiff,3),"\n", sep='')
+        X2 <- round(2*object2@logLik - 2*object@logLik, 3)
+        cat('\nModel 1: ')
+        print(object@Call)
+        cat('Model 2: ')
+        print(object2@Call)
+        cat('\n')
+        ret <- data.frame(Df = c(object@df, object2@df),
+                          AIC = c(object@AIC, object2@AIC),
+                          AICc = c(object@AICc, object2@AICc),
+                          BIC = c(object@BIC, object2@BIC), 
+                          SABIC = c(object@SABIC, object2@SABIC),
+                          logLik = c(object@logLik, object2@logLik),
+                          X2 = c('', X2),
+                          df = c('', abs(df)),
+                          p = c('', round(1 - pchisq(X2,abs(df)),3)))         
+        return(ret)
     }		
 )
 
