@@ -76,7 +76,7 @@ MHRM.group <- function(pars, constrain, PrepList, list, debug)
     redun_constr <- rep(FALSE, length(estpars)) 
     if(length(constrain) > 0){
         for(i in 1:length(constrain)){            
-            L[constrain[[i]], constrain[[i]]] <- 1/length(constrain[[i]]) 
+            L[constrain[[i]], constrain[[i]]] <- 1
             for(j in 2:length(constrain[[i]]))
                 redun_constr[constrain[[i]][j]] <- TRUE
         }
@@ -87,7 +87,13 @@ MHRM.group <- function(pars, constrain, PrepList, list, debug)
         stop('Constraint applied to fixed parameter(s) ', 
              paste(redindex[diag(L)[!estpars] > 0]), ' but should only be applied to 
                  estimated parameters. Please fix!')
-    }          
+    }  
+    #make sure constrained pars are equal    
+    tmp <- rowSums(L)
+    tmp[tmp == 0] <- 1
+    tmp <- matrix(1/tmp, length(longpars), length(longpars), byrow = TRUE)
+    tmp2 <- abs(diag(L) - 1)
+    longpars <- diag((tmp * L) * longpars) + tmp2 * longpars
     ####Big MHRM loop
     for(cycles in 1:(NCYCLES + BURNIN + SEMCYCLES))
     {     
@@ -121,6 +127,10 @@ MHRM.group <- function(pars, constrain, PrepList, list, debug)
                         pars[[g]][[i]]@par[length(pars[[g]][[i]]@par)] <- 1
                     if(pars[[g]][[i]]@par[length(pars[[g]][[i]]@par)-1] < 0) 
                         pars[[g]][[i]]@par[length(pars[[g]][[i]]@par)-1] <- 0
+                    if(pars[[g]][[i]]@par[length(pars[[g]][[i]]@par)-1] > .6){ 
+                        warning('lower bound parameter larger than .6 during estimation.')
+                        pars[[g]][[i]]@par[length(pars[[g]][[i]]@par)-1] <- .6
+                    }
                 }            
                 #apply sum(t) == 1 constraint for mcm
                 if(is(pars[[g]][[i]], 'mcm')){
@@ -209,6 +219,14 @@ MHRM.group <- function(pars, constrain, PrepList, list, debug)
             correction <- as.vector(inv.ave.h %*% grad)
             correction[correction > .5] <- 1
             correction[correction < -.5] <- -1
+            #prevent guessing/upper pars from moving more than .001 at all times
+            names(correction) <- names(estpars[estpars & !redun_constr])                   
+            tmp <- correction[names(correction) == 'g']
+            tmp[abs(tmp) > .001] <- sign(tmp[abs(tmp) > .001]) * .001/gamma
+            correction[names(correction) == 'g'] <- tmp       
+            tmp <- correction[names(correction) == 'u']
+            tmp[abs(tmp*gamma) > .001] <- sign(tmp[abs(tmp*gamma) > .001]) * .001/gamma
+            correction[names(correction) == 'u'] <- tmp
             longpars[estindex_unique] <- longpars[estindex_unique] + gamma*correction           
             if(length(constrain) > 0)
                 for(i in 1:length(constrain))
@@ -222,7 +240,7 @@ MHRM.group <- function(pars, constrain, PrepList, list, debug)
                 SEM.stores2[[cycles - BURNIN]] <- ave.h
             }	
             next
-        }	 
+        }	                 
         
         #Step 3. Update R-M step		
         Tau <- Tau + gamma*(ave.h - Tau)
@@ -238,8 +256,18 @@ MHRM.group <- function(pars, constrain, PrepList, list, debug)
             if(noninvcount == 5) 
                 stop('\nEstimation halted during stage 3, solution is unstable')
         }		
-        correction <- as.vector(inv.Tau %*% grad)
-        longpars[estindex_unique] <- longpars[estindex_unique] + gamma*correction           
+        correction <- as.vector(inv.Tau %*% grad) 
+        correction[gamma*correction > .25] <- .25/gamma
+        correction[gamma*correction < -.25] <- -.25/gamma
+        #prevent guessing/upper pars from moving more than .001 at all times
+        names(correction) <- names(estpars[estpars & !redun_constr])                   
+        tmp <- correction[names(correction) == 'g']
+        tmp[abs(tmp*gamma) > .001] <- sign(tmp[abs(tmp*gamma) > .001]) * .001/gamma
+        correction[names(correction) == 'g'] <- tmp
+        tmp <- correction[names(correction) == 'u']
+        tmp[abs(tmp*gamma) > .001] <- sign(tmp[abs(tmp*gamma) > .001]) * .001/gamma
+        correction[names(correction) == 'u'] <- tmp
+        longpars[estindex_unique] <- longpars[estindex_unique] + gamma*correction                   
         if(length(constrain) > 0)
             for(i in 1:length(constrain))
                 longpars[index %in% constrain[[i]][-1]] <- longpars[constrain[[i]][1]]
@@ -262,6 +290,7 @@ MHRM.group <- function(pars, constrain, PrepList, list, debug)
         info <- info + gamma*(as.matrix(Tau) - phi %*% t(phi) - info)		
     } ###END BIG LOOP       
     #Reload final pars list
+    if(cycles == NCYCLES + BURNIN + SEMCYCLES) converge <- 0
     if(list$USEEM) longpars <- list$startlongpars
     ind1 <- 1
     for(g in 1:ngroups){
@@ -274,6 +303,10 @@ MHRM.group <- function(pars, constrain, PrepList, list, debug)
                     pars[[g]][[i]]@par[length(pars[[g]][[i]]@par)] <- 1
                 if(pars[[g]][[i]]@par[length(pars[[g]][[i]]@par)-1] < 0) 
                     pars[[g]][[i]]@par[length(pars[[g]][[i]]@par)-1] <- 0
+                if(pars[[g]][[i]]@par[length(pars[[g]][[i]]@par)-1] > .6){ 
+                    warning('lower bound parameter larger than .6 during estimation.')
+                    pars[[g]][[i]]@par[length(pars[[g]][[i]]@par)-1] <- .6
+                }
             }
             #apply sum(t) == 1 constraint for mcm
             if(is(pars[[g]][[i]], 'mcm')){
@@ -303,7 +336,8 @@ MHRM.group <- function(pars, constrain, PrepList, list, debug)
             pars[[g]][[i]]@SEpar <- SE[ind1:ind2]
             ind1 <- ind2 + 1            
         }         
-    }    
+    }        
+    info <- nameInfoMatrix(info=info, correction=correction, L=L, npars=length(longpars))
     ret <- list(pars=pars, cycles = cycles - BURNIN - SEMCYCLES, info=as.matrix(info), 
                 longpars=longpars, converge=converge)
     ret    
