@@ -25,18 +25,10 @@
 #' item. For example, if for a 4 item test with two specific factors, the first
 #' specific factor loads on the first two items and the second specific factor
 #' on the last two, then the vector is \code{c(1,1,2,2)}.
-#' @param itemtype type of items to be modeled, declared as a vector for each item or a single value
-#' which will be repeated globally. The NULL default assumes that the items follow a graded or 2PL structure,
-#' however they may be changed to the following: '2PL', '3PL', '3PLu', 
-#' '4PL', 'graded', 'grsm', 'gpcm', 'nominal', 'mcm', 'PC2PL', and 'PC3PL', 1 and 2 parameter logistic, 
-#' 3 parameter logistic (lower asymptote and upper), 4 parameter logistic, graded response model, 
-#' rating scale graded response model, generalized partial credit model, nominal model, 
-#' multiple choice model, and 2-3PL partially compensatory model, respectively
-#' @param grsm.block an optional numeric vector indicating where the blocking should occur when using 
-#' the grsm, NA represents items that do not belong to the grsm block (other items that may be estimated
-#' in the test data). For example, to specify two blocks of 3 with a 2PL item for the last item:
-#' \code{grsm.block = c(rep(1,3), rep(2,3), NA)}. If NULL the all items are assumed to be within the same 
-#' group and therefore have the same number of item categories
+#' @param itemtype see \code{\link{mirt}} for details
+#' @param grsm.block see \code{\link{mirt}} for details
+#' @param rsm.block see \code{\link{mirt}} for details
+#' @param calcNull logical; calculate the Null model for fit statics (e.g., TLI)?
 #' @param guess fixed pseudo-guessing parameter. Can be entered as a single
 #' value to assign a global value or may be entered as a numeric vector for
 #' each item of length \code{ncol(data)}.
@@ -46,22 +38,9 @@
 #' @param SE logical, estimate the standard errors? Calls the MHRM subroutine for a stochastic approximation
 #' @param SEtol tollerance value used to stop the MHRM estimation when \code{SE = TRUE}. Lower values
 #' will take longer but may be more stable for computing the information matrix
-#' @param constrain a list of user declared equality constraints. To see how to define the
-#' parameters correctly use \code{pars = 'values'} initially to see how the parameters are labeled.
-#' To constrain parameters to be equal create a list with separate concatenated vectors signifying which
-#' parameters to constrain. For example, to set parameters 1 and 5 equal, and also set parameters 2, 6, and 10 equal
-#' use \code{constrain = list(c(1,5), c(2,6,10))}
-#' @param parprior a list of user declared prior item probabilities. To see how to define the
-#' parameters correctly use \code{pars = 'values'} initially to see how the parameters are labeled.
-#' Can define either normal (normally for slopes and intercepts) or beta (for guessing and upper bounds) prior
-#' probabilities. To specify a prior the form is c('priortype', ...), where normal priors 
-#' are \code{parprior = list(c(parnumbers, 'norm', mean, sd))} and betas are 
-#' \code{parprior = list(c(parnumbers, 'beta', alpha, beta))}
-#' \code{parprior = list(c(parnumber, 'beta', alpha, beta))}. 
-#' @param pars a data.frame with the structure of how the starting values, parameter numbers, and estimation
-#' logical values are defined. The user may observe how the model defines the values by using \code{pars = 
-#' 'values'}, and this object can in turn be modified and input back into the estimation with \code{pars = 
-#' mymodifiedpars}
+#' @param constrain see \code{\link{mirt}} for details
+#' @param parprior see \code{\link{mirt}} for details
+#' @param pars see \code{\link{mirt}} for details
 #' @param D a numeric value used to adjust the logistic metric to be more similar to a normal
 #' cumulative density curve. Default is 1.702
 #' @param prev.cor uses a previously computed correlation matrix to be used to
@@ -69,18 +48,12 @@
 #' @param quadpts number of quadrature points per dimension. 
 #' @param verbose logical; print observed log-likelihood value at each iteration?
 #' @param debug logical; turn on debugging features?
-#' @param technical a list containing lower level technical parameters for estimation
-#' \describe{ 
-#' \item{MAXQUAD}{maximum number of quadratures; default 10000}
-#' \item{MSTEPMAXIT}{number of M-step iterations; default 25}
-#' \item{TOL}{EM convergence threshold; default .001}
-#' \item{NCYCLES}{maximum number of EM cycles; default 300}
-#' }
+#' @param technical see \code{\link{mirt}} for details
 #' @param ... additional arguments to be passed
 #' @author Phil Chalmers \email{rphilip.chalmers@@gmail.com}
 #' @seealso
 #' \code{\link{expand.table}}, \code{\link{key2binary}}, \code{\link{confmirt}},
-#' \code{\link{fscores}}, \code{\link{multipleGroup}}, \code{\link{wald}}
+#' \code{\link{fscores}}, \code{\link{multipleGroup}}, \code{\link{wald}}, \code{\link{fitIndices}}
 #' @references
 #' 
 #' Chalmers, R., P. (2012). mirt: A Multidimensional Item Response Theory
@@ -98,9 +71,8 @@
 #' @keywords models
 #' @usage
 #' bfactor(data, model, itemtype = NULL, guess = 0, upper = 1, SE = FALSE, SEtol = .001, pars = NULL,
-#' constrain = NULL, parprior = NULL,
-#' prev.cor = NULL, quadpts = 20, grsm.block = NULL, D = 1.702, verbose = FALSE, debug = FALSE, 
-#' technical = list(), ...)
+#' constrain = NULL, parprior = NULL, calcNull = TRUE, prev.cor = NULL, quadpts = 20, grsm.block = NULL, 
+#' rsm.block = NULL, D = 1.702, verbose = FALSE, debug = FALSE, technical = list(), ...)
 #' 
 #'
 #' @export bfactor
@@ -167,8 +139,8 @@
 #'     }
 #' 
 bfactor <- function(data, model, itemtype = NULL, guess = 0, upper = 1, SE = FALSE, SEtol = .001,
-                    pars = NULL, constrain = NULL, parprior = NULL, prev.cor = NULL, quadpts = 20, 
-                    grsm.block = NULL, D = 1.702, verbose = FALSE, debug = FALSE, 
+                    pars = NULL, constrain = NULL, parprior = NULL, calcNull = TRUE, prev.cor = NULL, 
+                    quadpts = 20, grsm.block = NULL, rsm.block = NULL, D = 1.702, verbose = FALSE, debug = FALSE, 
                     technical = list(), ...)
 {         
     if(debug == 'Main') browser()
@@ -176,9 +148,9 @@ bfactor <- function(data, model, itemtype = NULL, guess = 0, upper = 1, SE = FAL
     mod <- ESTIMATION(data=data, model=model, group=rep('all', nrow(data)), 
                       itemtype=itemtype, guess=guess, upper=upper, grsm.block=grsm.block,
                       pars=pars, method = 'EM', constrain=constrain, SE = SE, SEtol=SEtol,
-                      parprior=parprior, quadpts=quadpts, D=D,
+                      parprior=parprior, quadpts=quadpts, D=D, rsm.block=rsm.block,
                       technical = technical, debug = debug, verbose = verbose, 
-                      BFACTOR = TRUE, ...)
+                      BFACTOR = TRUE, calcNull=calcNull, ...)
     if(is(mod, 'ConfirmatoryClass') || is(mod, 'MultipleGroupClass'))
         mod@Call <- Call
     return(mod)
