@@ -4,8 +4,11 @@
 #' maximum-likelihood factor analysis model to dichotomous and polytomous
 #' data under the item response theory paradigm using Cai's (2010)
 #' Metropolis-Hastings Robbins-Monro algorithm. Fits univariate and multivariate Rasch, 
-#' 1-4PL, graded, (generalized) partial credit, nominal, multiple choice, and partially-compensatory models, 
-#' potentially with polynomial and product constructed latent traits.
+#' 1-4PL, graded, (generalized) partial credit, nominal, multiple choice, graded rating scale, Rasch rating scale,
+#' and partially-compensatory models, potentially with polynomial and product constructed latent traits. 
+#' Models may also contain 'explanatory' 
+#' person or item level predictors, though these can only be included by using the 
+#' \code{\link{mixedmirt}} function.
 #'  
 #' \code{confmirt} follows a confirmatory and exploratory item factor analysis strategy that
 #' uses a stochastic version of maximum likelihood estimation described by Cai
@@ -73,36 +76,16 @@
 #' \code{abs(printvalue)}. The default (NULL) prints all response patterns
 #' @param verbose logical; display iteration history during estimation?
 #' @param draws the number of Monte Carlo draws to estimate the log-likelihood
+#' @param calcNull logical; calculate the Null model for fit statics (e.g., TLI)?
 #' @param restype type of residuals to be displayed. Can be either \code{'LD'}
 #' for a local dependence matrix (Chen & Thissen, 1997) or \code{'exp'} for the
 #' expected values for the frequencies of every response pattern
-#' @param itemtype type of items to be modeled, declared as a vector for each item or a single value
-#' which will be repeated globally. The NULL default assumes that the items follow a graded or 2PL structure,
-#' however they may be changed to the following: 'Rasch', '1PL', '2PL', '3PL', '3PLu', 
-#' '4PL', 'graded', 'grsm', 'gpcm', 'nominal', 'mcm', 'PC2PL', and 'PC3PL', for the Rasch/partial credit, 1 and 2 parameter logistic, 
-#' 3 parameter logistic (lower asymptote and upper), 4 parameter logistic, graded response model, 
-#' rating scale graded response model, generalized partial credit model, nominal model, 
-#' multiple choice model, and 2-3PL partially compensatory model, respectively
-#' @param grsm.block an optional numeric vector indicating where the blocking should occur when using 
-#' the grsm, NA represents items that do not belong to the grsm block (other items that may be estimated
-#' in the test data). For example, to specify two blocks of 3 with a 2PL item for the last item:
-#' \code{grsm.block = c(rep(1,3), rep(2,3), NA)}. If NULL the all items are assumed to be within the same 
-#' group and therefore have the same number of item categories
-#' @param constrain a list of user declared equality constraints. To see how to define the
-#' parameters correctly use \code{pars = 'values'} initially to see how the parameters are labeled.
-#' To constrain parameters to be equal create a list with separate concatenated vectors signifying which
-#' parameters to constrain. For example, to set parameters 1 and 5 equal, and also set parameters 2, 6, and 10 equal
-#' use \code{constrain = list(c(1,5), c(2,6,10))}
-#' @param parprior a list of user declared prior item probabilities. To see how to define the
-#' parameters correctly use \code{pars = 'values'} initially to see how the parameters are labeled.
-#' Can define either normal (normally for slopes and intercepts) or beta (for guessing and upper bounds) prior
-#' probabilities. To specify a prior the form is c('priortype', ...), where normal priors 
-#' are \code{parprior = list(c(parnumbers, 'norm', mean, sd))} and betas are 
-#' \code{parprior = list(c(parnumbers, 'beta', alpha, beta))}
-#' @param pars a data.frame with the structure of how the starting values, parameter numbers, and estimation
-#' logical values are defined. The user may observe how the model defines the values by using \code{pars = 
-#' 'values'}, and this object can in turn be modified and input back into the estimation with \code{pars = 
-#' mymodifiedpars}
+#' @param itemtype see \code{\link{mirt}} for details
+#' @param grsm.block see \code{\link{mirt}} for details
+#' @param rsm.block see \code{\link{mirt}} for details
+#' @param constrain see \code{\link{mirt}} for details
+#' @param parprior see \code{\link{mirt}} for details
+#' @param pars see \code{\link{mirt}} for details
 #' @param debug logical; turn on debugging features?
 #' @param object an object of class \code{ConfirmatoryClass}
 #' @param object2 an object of class \code{ConfirmatoryClass}
@@ -145,7 +128,8 @@
 #' @seealso
 #' \code{\link{expand.table}}, \code{\link{key2binary}}, \code{\link{simdata}},
 #' \code{\link{fscores}}, \code{\link{confmirt.model}}, \code{\link{wald}}, 
-#' \code{\link{multipleGroup}}, \code{\link{itemplot}}
+#' \code{\link{multipleGroup}}, \code{\link{itemplot}}, \code{\link{fitIndices}}, 
+#' \code{\link{mixedmirt}}, \code{\link{testinfo}}, \code{\link{iteminfo}},
 #' @references
 #' 
 #' Cai, L. (2010a). High-Dimensional exploratory item factor analysis by a
@@ -167,7 +151,7 @@
 #' @keywords models
 #' @usage 
 #' confmirt(data, model, itemtype = NULL, guess = 0, upper = 1, pars = NULL, 
-#' constrain = NULL, parprior = NULL, grsm.block = NULL, verbose = TRUE, 
+#' constrain = NULL, parprior = NULL, calcNull = TRUE, grsm.block = NULL, rsm.block = NULL, verbose = TRUE, 
 #' draws = 2000, debug = FALSE, rotate = 'varimax', Target = NULL, D = 1.702, 
 #' technical = list(),  ...)
 #' 
@@ -274,16 +258,16 @@
 #' }
 #' 
 confmirt <- function(data, model, itemtype = NULL, guess = 0, upper = 1, pars = NULL, 
-                     constrain = NULL, parprior = NULL, grsm.block = NULL, verbose = TRUE, 
-                     draws = 2000, debug = FALSE, rotate = 'varimax', Target = NULL, 
+                     constrain = NULL, parprior = NULL, calcNull = TRUE, grsm.block = NULL, rsm.block = NULL, 
+                     verbose = TRUE, draws = 2000, debug = FALSE, rotate = 'varimax', Target = NULL, 
                      D = 1.702, technical = list(),  ...)
 {   
     if(debug == 'Main') browser()
     Call <- match.call()    
     mod <- ESTIMATION(data=data, model=model, group = rep('all', nrow(data)), itemtype=itemtype, 
-                      guess=guess, upper=upper, grsm.block=grsm.block, D=D,
+                      guess=guess, upper=upper, grsm.block=grsm.block, D=D, calcNull=calcNull,
                       pars=pars, constrain=constrain, parprior=parprior, verbose=verbose, 
-                      draws=draws, debug=debug, technical = list(),  ...)
+                      rsm.block=rsm.block, draws=draws, debug=debug, technical = list(),  ...)
     if(is(mod, 'ExploratoryClass') || is(mod, 'ConfirmatoryClass'))
         mod@Call <- Call
     return(mod)    
