@@ -1,18 +1,14 @@
 #' Full information maximum likelihood estimation of multidimensional IRT models
 #' 
 #' Analysis of dichotomous and polytomous response data using latent
-#' trait models under the Item Response Theory paradigm. Includes univariate
-#' and multivariate one-, two-, three-, and four-parameter logistic models,
-#' graded response models, generalized partial credit models, nominal models,
-#' multiple choice models, and multivariate partially-compensatory models.
-#' These can be used in an exploratory or confirmatory manner with optional
-#' user defined linear constraints. Exploratory models can be estimated via
-#' quadrature or stochastic methods, a generalized confirmatory bi-factor
+#' trait models under the Item Response Theory paradigm. Exploratory models can be 
+#' estimated via quadrature or stochastic methods, a generalized confirmatory bi-factor
 #' analysis is included, and confirmatory models can be fit with a
 #' Metropolis-Hastings Robbins-Monro algorithm which can include polynomial or
-#' product constructed latent traits. Additionally, multiple group analysis may
-#' be performed for unidimensional or multidimensional item response models for
-#' detecting differential item functioning.
+#' product constructed latent traits. Multiple group analysis
+#' and mixed effects designs may be performed for unidimensional or
+#' multidimensional item response models for detecting differential item
+#' functioning and modelling item and person covariates.
 #' 
 #' Users interested in the most recent version of this package can visit 
 #' \code{https://github.com/philchalmers/mirt} and follow the instructions 
@@ -81,3 +77,33 @@ NULL
 #'
 #' @keywords data
 NULL
+
+#' Description of LSAT6 data
+#' 
+#' Data from Thissen (1982); contains 5 dichotomously scored 
+#' items obtained from the Law School Admissions Test, section 6.
+#' 
+#' 
+#' @name LSAT6
+#' @docType data
+#' @author Phil Chalmers \email{rphilip.chalmers@@gmail.com}
+#' @references
+#' Thissen, D. (1982). Marginal maximum likelihood estimation for the one-parameter logistic model.
+#' \emph{Psychometrika, 47}, 175-186.
+#'
+#' @keywords data
+NULL
+
+#' Description of deAyala data
+#' 
+#' Mathematics data from de Ayala (2009; pg. 14); 5 item dataset in table format.
+#' 
+#' 
+#' @name deAyala
+#' @docType data
+#' @author Phil Chalmers \email{rphilip.chalmers@@gmail.com}
+#' @references
+#' de Ayala, R. J. (2009). \emph{The theory and practice of item response theory}. Guilford Press.
+#'
+#' @keywords data
+NULL
