@@ -4,7 +4,10 @@
 #' \code{mirt} fits an unconditional maximum likelihood factor analysis model
 #' to dichotomous and polytomous data under the item response theory paradigm. 
 #' Fits univariate and multivariate Rasch, 1-4PL, graded, (generalized) partial credit, 
-#' nominal, multiple choice, and partially compenatory models using the EM algorithm.
+#' nominal, multiple choice, graded rating scale, Rasch rating scale, 
+#' and partially compenatory models using the EM algorithm. Models may also contain 'explanatory' 
+#' person or item level predictors, though these can only be included by using the 
+#' \code{\link{mixedmirt}} function.
 #' 
 #' \code{mirt} follows the item factor analysis strategy by marginal maximum
 #' likelihood estimation (MML) outlined in Bock and Aiken (1981), Bock,
@@ -36,7 +39,9 @@
 #' variances of the latent factors are automatically fixed to 1 to help
 #' facilitate model identification. All parameters may be fixed to constant
 #' values or set equal to other parameters using the appropriate declarations.
-#' If the model is confirmatory then the returned class will be 'ConfirmatoryClass'.
+#' If the model is confirmatory then the returned class will be 'ConfirmatoryClass'. Confirmatory
+#' models may also contain 'explanatory' person or item level predictors, though including predictors
+#' is limited only to the \code{\link{mixedmirt}} function.
 #' 
 #' @section Exploratory IRT:
 #' 
@@ -73,15 +78,17 @@
 #' @param itemtype type of items to be modeled, declared as a vector for each item or a single value
 #' which will be repeated globally. The NULL default assumes that the items follow a graded or 2PL structure,
 #' however they may be changed to the following: 'Rasch', '1PL', '2PL', '3PL', '3PLu', 
-#' '4PL', 'graded', 'grsm', 'gpcm', 'nominal', 'mcm', 'PC2PL', and 'PC3PL', for the Rasch/partial credit, 1 and 2 parameter logistic, 
+#' '4PL', 'graded', 'grsm', 'gpcm', 'rsm', 'nominal', 'mcm', 'PC2PL', and 'PC3PL', 
+#' for the Rasch/partial credit, 1 and 2 parameter logistic, 
 #' 3 parameter logistic (lower asymptote and upper), 4 parameter logistic, graded response model, 
-#' rating scale graded response model, generalized partial credit model, nominal model, 
+#' rating scale graded response model, generalized partial credit model, Rasch rating scale model, nominal model, 
 #' multiple choice model, and 2-3PL partially compensatory model, respectively 
 #' @param grsm.block an optional numeric vector indicating where the blocking should occur when using 
 #' the grsm, NA represents items that do not belong to the grsm block (other items that may be estimated
 #' in the test data). For example, to specify two blocks of 3 with a 2PL item for the last item:
 #' \code{grsm.block = c(rep(1,3), rep(2,3), NA)}. If NULL the all items are assumed to be within the same 
 #' group and therefore have the same number of item categories
+#' @param rsm.block same as \code{grsm.block}, but for \code{'rsm'} blocks
 #' @param SE logical, estimate the standard errors? Calls the MHRM subroutine for a stochastic approximation
 #' @param SEtol tollerance value used to stop the MHRM estimation when \code{SE = TRUE}. Lower values
 #' will take longer but may be more stable for computing the information matrix
@@ -116,6 +123,7 @@
 #' logical values are defined. The user may observe how the model defines the values by using \code{pars = 
 #' 'values'}, and this object can in turn be modified and input back into the estimation with \code{pars = 
 #' mymodifiedpars}
+#' @param calcNull logical; calculate the Null model for fit statics (e.g., TLI)?
 #' @param quadpts number of quadrature points per dimension
 #' @param printvalue a numeric value to be specified when using the \code{res='exp'}
 #' option. Only prints patterns that have standardized residuals greater than 
@@ -178,6 +186,8 @@
 #' Depending on the model \eqn{u} may be equal to 1 and \eqn{g} may be equal to 0. 
 #' \deqn{P(x = 1|\theta, \psi) = g + \frac{(u - g)}{1 + exp(-D * 
 #' (a_1 * \theta_1 + a_2 * \theta_2 + d))}} 
+#' For the 1PL model the number of factors must equal 1, and all the \eqn{a_1} values are constrained
+#' to be equal accross all items.
 #' }
 #' \item{graded}{
 #' The graded model consists of sequential 2PL models, and here \eqn{k} is 
@@ -197,6 +207,14 @@
 #' \deqn{P(x = k | \theta, \psi) = \frac{exp(-D * ak_k * (a_1 * \theta_1 + a_2 * \theta_2) + d_k)}
 #' {\sum_i^k exp(-D * ak_k * (a_1 * \theta_1 + a_2 * \theta_2) + d_k)}}
 #' }
+#' \item{rsm}{
+#' A more constrained version of the generalized partial credit model where the spacing is equal 
+#' accross item blocks and only adjusted by a single 'difficulty' parameter (c), and the discrimination
+#' parameter is constrained to be a constant. Note that this is analogous to the relationship 
+#' between the graded model and the grsm (with an additional constraint regarding the discrimination 
+#' parameters; the discrimination constraint can, however, be relaxed by adjusting the starting values
+#' manually).
+#' }
 #' \item{mcm}{For identification \eqn{ak_0 = d_0 = 0} and \eqn{\sum_0^k t_k = 1}.
 #' \deqn{P(x = k | \theta, \psi) = C_0 (\theta) * t_k  + (1 - C_0 (\theta)) * 
 #' \frac{exp(-D * ak_k * (a_1 * \theta_1 + a_2 * \theta_2) + d_k)}  
@@ -214,8 +232,9 @@
 #' @author Phil Chalmers \email{rphilip.chalmers@@gmail.com}
 #' @seealso
 #' \code{\link{expand.table}}, \code{\link{key2binary}}, 
-#' \code{\link{confmirt}}, \code{\link{bfactor}}, \code{\link{multipleGroup}}, \code{\link{wald}}
-#' \code{\link{itemplot}}, \code{\link{fscores}}
+#' \code{\link{confmirt}}, \code{\link{bfactor}}, \code{\link{multipleGroup}}, \code{\link{mixedmirt}}, 
+#' \code{\link{wald}}, \code{\link{itemplot}}, \code{\link{fscores}}, \code{\link{fitIndices}}, 
+#' \code{\link{extract.item}}, \code{\link{iteminfo}}, \code{\link{testinfo}}
 #' 
 #' @references
 #' 
@@ -247,9 +266,9 @@
 #' @keywords models
 #' @usage 
 #' mirt(data, model, itemtype = NULL, guess = 0, upper = 1, SE = FALSE, SEtol = .001, pars = NULL, 
-#' constrain = NULL, parprior = NULL, rotate = 'varimax', Target = NaN, 
-#' prev.cor = NULL, quadpts = NULL, grsm.block = NULL, D = 1.702, verbose = FALSE, debug = FALSE, 
-#' technical = list(), ...)
+#' constrain = NULL, parprior = NULL, calcNull = TRUE, rotate = 'varimax', Target = NaN, 
+#' prev.cor = NULL, quadpts = NULL, grsm.block = NULL, rsm.block = NULL, D = 1.702, verbose = FALSE, 
+#' debug = FALSE, technical = list(), ...)
 #' 
 #' \S4method{summary}{ExploratoryClass}(object, rotate = '', Target = NULL, suppress = 0, digits = 3, 
 #' verbose = TRUE, ...)
@@ -310,10 +329,11 @@
 #' #Constrain all slopes to be equal
 #' #first obtain parameter index
 #' values <- mirt(Science,1, pars = 'values') 
-#' values #note that slopes are numbered 1,5,9,13
-#' (pmod1_equalslopes <- mirt(Science, 1, constrain = list(c(1,5,9,13))))
+#' values #note that slopes are numbered 1,5,9,13, or index with values$parnum[values$name == 'a1']
+#' (pmod1_equalslopes <- mirt(Science, 1, constrain = list(c(1,5,9,13)))) 
 #' coef(pmod1_equalslopes)
-#' 
+#' anova(pmod1_equalslopes, pmod1) #significantly worse fit with all criteria
+#'
 #' pmod2 <- mirt(Science, 2)
 #' summary(pmod2)
 #' residuals(pmod2)
@@ -321,8 +341,17 @@
 #' itemplot(pmod2, 1)
 #' anova(pmod1, pmod2)
 #' 
+#' #unidimensional fit with a generalized partial credit and nominal model
+#' (gpcmod <- mirt(Science, 1, 'gpcm'))
+#' coef(gpcmod)
+#' (nomod <- mirt(Science, 1, 'nominal'))
+#' anova(gpcmod, nomod)
+#' itemplot(nomod, 3)
+#' 
 #' 
 #' ###########
+#' #empirical dimensionality testing that includes 'guessing'
+#' 
 #' data(SAT12)
 #' data <- key2binary(SAT12,
 #'   key = c(1,4,5,2,3,1,2,1,3,1,2,4,2,1,5,3,4,4,1,4,3,3,4,1,3,5,1,3,1,5,4,5))
@@ -350,6 +379,7 @@
 #' #graded rating scale example
 #' 
 #' #make some data
+#' set.seed(1234)
 #' a <- matrix(rep(1/1.702, 10))
 #' d <- matrix(c(1,0.5,-.5,-1), 10, 4, byrow = TRUE)
 #' c <- seq(-1, 1, length.out=10)
@@ -360,15 +390,15 @@
 #' sv[,5] <- c(as.vector(t(cbind(a,d,c))),0,1) 
 #'
 #' mod1 <- mirt(data, 1)
-#' mod2 <- mirt(data, 1, itemtype = 'grsm', verbose = TRUE, pars = sv)
+#' mod2 <- mirt(data, 1, itemtype = 'grsm', verbose = TRUE, pars = sv, calcNull = FALSE)
 #' coef(mod2)
 #' anova(mod2, mod1) #not sig, mod2 should be prefered 
 #' }
 #' 
 mirt <- function(data, model, itemtype = NULL, guess = 0, upper = 1, SE = FALSE, SEtol = .001,
-                 pars = NULL, constrain = NULL, parprior = NULL, rotate = 'varimax', Target = NaN, 
-                 prev.cor = NULL, quadpts = NULL, grsm.block = NULL, D = 1.702, verbose = FALSE, 
-                 debug = FALSE, technical = list(), ...)
+                 pars = NULL, constrain = NULL, parprior = NULL, calcNull = TRUE, rotate = 'varimax', 
+                 Target = NaN, prev.cor = NULL, quadpts = NULL, grsm.block = NULL, rsm.block = NULL, 
+                 D = 1.702, verbose = FALSE, debug = FALSE, technical = list(), ...)
 {   
     if(debug == 'Main') browser()
     Call <- match.call()    
@@ -376,7 +406,8 @@ mirt <- function(data, model, itemtype = NULL, guess = 0, upper = 1, SE = FALSE,
                       itemtype=itemtype, guess=guess, upper=upper, grsm.block=grsm.block,
                       pars=pars, method = 'EM', constrain=constrain, SE=SE, SEtol=SEtol,
                       parprior=parprior, quadpts=quadpts, rotate=rotate, Target=Target, D=D,
-                      technical = technical, debug = debug, verbose = verbose, ...)
+                      rsm.block=rsm.block, technical=technical, debug=debug, verbose=verbose,
+                      calcNull=calcNull, ...)
     if(is(mod, 'ExploratoryClass') || is(mod, 'ConfirmatoryClass'))
         mod@Call <- Call
     return(mod)    
