@@ -1,9 +1,10 @@
 model.elements <- function(model, factorNames, itemtype, nfactNames, nfact, J, K, fulldata, 
                            itemloc, data, N, guess, upper, itemnames, exploratory, constrain, 
-                           startvalues, freepars, parprior, parnumber, BFACTOR = FALSE, D, debug)
+                           startvalues, freepars, parprior, parnumber, BFACTOR = FALSE, D, mixedlist,
+                           debug)
 {       
     if(debug == 'model.elements') browser()
-    hasProdTerms <- ifelse(nfact == nfactNames, FALSE, TRUE)
+    hasProdTerms <- ifelse(nfact == nfactNames, FALSE, TRUE)    
     prodlist <- NULL
     if(hasProdTerms){
         tmp <- factorNames[grepl('\\(',factorNames)]
@@ -85,11 +86,17 @@ model.elements <- function(model, factorNames, itemtype, nfactNames, nfact, J, K
         u[u < .001 ] <- .2
         cs <- sqrt(u)
         lambdas <- loads/cs * (1.702/D)                  
-    }        
+    }
+    if(!is.null(mixedlist)){
+        covdata <- mixedlist$covdata
+        Theta <- matrix(0, N, nrow(model))
+        colnames(Theta) <- model[ ,1]
+        mixedlist$Theta <- Theta        
+    }
     ret <- LoadPars(itemtype=itemtype, itemloc=itemloc, lambdas=lambdas, zetas=zetas, guess=guess, upper=upper,
                     fulldata=fulldata, J=J, K=K, nfact=nfact+length(prodlist), constrain=constrain, 
                     startvalues=startvalues, freepars=freepars, parprior=parprior, parnumber=parnumber,
-                    estLambdas=estlam, BFACTOR=BFACTOR, D=D, debug=debug)      
+                    estLambdas=estlam, BFACTOR=BFACTOR, D=D, mixedlist=mixedlist, debug=debug)      
     ret[[length(ret) + 1]] <- LoadGroupPars(gmeans=gmeans, gcov=gcov, estgmeans=estgmeans, 
                                             estgcov=estgcov, parnumber=attr(ret, 'parnumber'),
                                             startvalues=startvalues, freepars=freepars, parprior=parprior,
